unit Adventure.Form.FMX;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs, FMX.Edit,
  ksTypes, ksChatView, ksTableView,
  {$IFDEF WINDOWS}
  SpeechLib_TLB,
  {$ENDIF}
  DosAdventure;

type
  TAdventureForm = class(TForm)
    ksChatView1: TksChatView;
    procedure FormCreate(Sender: TObject);
    procedure ksChatView1SendButtonClick(Sender: TObject; var ABubble: TksChatBubbleInfo; var AAllowPost: Boolean);
  private
    FDosAdventure: TDosAdventure;
    {$IFDEF WINDOWS}
    FSpVoice: TSpVoice;
    {$ENDIF}
    procedure AdventureResponse(Sender: TObject; const AResponse: string);
    function IsRunning: Boolean;
    procedure PrepareInput;
    function ReformatReponse(const AText: string): string;
    procedure SendText(const AText: string);
    procedure Start;
    procedure UpdateData(const AText: string);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Speak(const AText: string);
  end;

var
  AdventureForm: TAdventureForm;

implementation

uses
  System.UIConsts, System.Threading, System.StrUtils;

{$R *.fmx}

type
  TksChatViewHelper = class helper for TksChatView
  public
    function GetEdit: TEdit;
    property Edit: TEdit read GetEdit;
  end;

function TksChatViewHelper.GetEdit: TEdit;
var
  I: Integer;
begin
  for I := 0 to ComponentCount - 1 do begin
    if Components[I] is TksChatViewEdit then begin
      Exit(TksChatViewEdit(Components[I]).Edit);
    end;
  end;
  result := nil;
end;

constructor TAdventureForm.Create(AOwner: TComponent);
begin
  inherited;
  FDosAdventure := TDosAdventureFactory.CreateDosAdventure;
  FDosAdventure.OnNotifyResponse := AdventureResponse;
  {$IFDEF WINDOWS}
  FSpVoice := TSpVoice.Create(Self);
  {$ENDIF}
end;

destructor TAdventureForm.Destroy;
begin
  FDosAdventure.Free;
  inherited Destroy;
end;

procedure TAdventureForm.AdventureResponse(Sender: TObject; const AResponse: string);
begin
  UpdateData(AResponse);
end;

procedure TAdventureForm.FormCreate(Sender: TObject);
var
  P: TPoint;
begin
  P := Screen.Displays[1].WorkareaRect.CenterPoint;
  Left := P.X - Width div 2;
  Top := P.Y - Height div 2;
  PrepareInput;
end;

function TAdventureForm.IsRunning: Boolean;
begin
  result := (FDosAdventure <> nil) and FDosAdventure.IsRunning;
end;

procedure TAdventureForm.ksChatView1SendButtonClick(Sender: TObject; var ABubble: TksChatBubbleInfo;
  var AAllowPost: Boolean);
begin
  if IsRunning then begin
    AAllowPost := true;
    SendText(ABubble.Text);
  end
  else begin
    AAllowPost := false;
    Start;
  end;
end;

procedure TAdventureForm.PrepareInput;
begin
  if IsRunning then begin
    ksChatView1.ButtonText := 'Send';
    ksChatView1.Edit.Enabled := true;
    ksChatView1.Edit.SetFocus;
  end
  else begin
    ksChatView1.ButtonText := 'Start';
    ksChatView1.Edit.Enabled := false;
  end;
end;

function TAdventureForm.ReformatReponse(const AText: string): string;
var
  lst: TStringList;
begin
  lst := TStringList.Create;
  try
    lst.Text := AText;
    { Ist die letzte Zeile eine Eingabeaufforderung? }
    if (lst.Count > 0) and MatchStr(lst[lst.Count - 1], ['* ', '** ']) then begin
      lst.Delete(lst.Count - 1);
    end;
    result := lst.Text;
  finally
    lst.Free;
  end;
end;

procedure TAdventureForm.SendText(const AText: string);
begin
  FDosAdventure.SendText(AText);
end;

procedure TAdventureForm.Speak(const AText: string);
var
  S: string;
begin
  S := ReformatReponse(AText).Replace(sLineBreak, ' ', [rfReplaceAll]);
  {$IFDEF WINDOWS}
  FSpVoice.Speak(S, SVSFlagsAsync)
  {$ENDIF}
end;

procedure TAdventureForm.Start;
begin
  ksChatView1.Clear;
  FDosAdventure.Start;
end;

procedure TAdventureForm.UpdateData(const AText: string);
begin
  Speak(AText);
  ksChatView1.AddChatBubble(ReformatReponse(AText), ksCbpRight, claBeige, claBlack, nil);
  PrepareInput;
end;

end.
