unit Adventure.Form.VCL;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, System.Actions,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ExtCtrls, Vcl.ActnList,
  DosAdventure, SpeechLib_TLB;

type
  TAdventureForm = class(TForm)
    ActionList1: TActionList;
    actSend: TAction;
    Panel1: TPanel;
    Button1: TButton;
    Edit1: TEdit;
    Panel2: TPanel;
    Memo1: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure actSendExecute(Sender: TObject);
    procedure actSendUpdate(Sender: TObject);
  private
    FDosAdventure: TDosAdventure;
    FSpVoice: TSpVoice;
    procedure AddToLastLine(const AText: string);
    procedure AdventureResponse(Sender: TObject; const AResponse: string);
    function IsRunning: Boolean;
    function ReformatReponse(const AText: string): string;
    procedure SendText(const AText: string);
    procedure Start;
    procedure UpdateData(const AText: string);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Speak(const AText: string);
  end;

var
  AdventureForm: TAdventureForm;

implementation

uses
  System.StrUtils;

{$R *.dfm}

constructor TAdventureForm.Create(AOwner: TComponent);
begin
  inherited;
  FDosAdventure := TDosAdventureFactory.CreateDosAdventure;
  FDosAdventure.OnNotifyResponse := AdventureResponse;
  FSpVoice := TSpVoice.Create(Self);
end;

destructor TAdventureForm.Destroy;
begin
  FDosAdventure.Free;
  inherited Destroy;
end;

procedure TAdventureForm.FormCreate(Sender: TObject);
var
  P: TPoint;
begin
  P := Screen.Monitors[1].WorkareaRect.CenterPoint;
  Left := P.X - Width div 2;
  Top := P.Y - Height div 2;
end;

procedure TAdventureForm.actSendExecute(Sender: TObject);
begin
  if IsRunning then begin
    SendText(Edit1.Text);
  end
  else begin
    Start;
  end;
end;

procedure TAdventureForm.actSendUpdate(Sender: TObject);
begin
  actSend.Caption := IfThen(IsRunning, 'Send', 'Start');
  Edit1.Enabled := IsRunning;
end;

procedure TAdventureForm.AddToLastLine(const AText: string);
begin
  if Memo1.Lines.Count > 0 then begin
    Memo1.Lines[Memo1.Lines.Count - 1] := Memo1.Lines[Memo1.Lines.Count - 1] + AText;
  end
  else begin
    Memo1.Lines.Add(AText);
  end;
end;

procedure TAdventureForm.AdventureResponse(Sender: TObject; const AResponse: string);
begin
  UpdateData(AResponse);
end;

function TAdventureForm.IsRunning: Boolean;
begin
  Result := (FDosAdventure <> nil) and FDosAdventure.IsRunning;
end;

function TAdventureForm.ReformatReponse(const AText: string): string;
var
  lst: TStringList;
begin
  lst := TStringList.Create;
  try
    lst.Text := AText;
    { Ist die letzte Zeile eine Eingabeaufforderung? }
    if (lst.Count > 0) and MatchStr(lst[lst.Count - 1], ['* ', '** ']) then begin
      lst.Delete(lst.Count - 1);
    end;
    result := lst.Text;
  finally
    lst.Free;
  end;
end;

procedure TAdventureForm.SendText(const AText: string);
begin
  AddToLastLine(AText);
  FDosAdventure.SendText(AText);
end;

procedure TAdventureForm.Speak(const AText: string);
var
  S: string;
begin
  S := ReformatReponse(AText).Replace(sLineBreak, ' ', [rfReplaceAll]);
  FSpVoice.Speak(S, SVSFlagsAsync)
end;

procedure TAdventureForm.Start;
begin
  Memo1.Lines.Clear;
  FDosAdventure.Start;
end;

procedure TAdventureForm.UpdateData(const AText: string);
begin
  Speak(AText);
  Memo1.Lines.Add(AText);
  if Edit1.Enabled then begin
    Edit1.Clear;
    Edit1.SetFocus;
  end;
end;

end.
